/*
 * Copyright (c) 2008 Hans - Joachim Borchers
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <avr\interrupt.h>
#include "typedef.h"
#include "sysdef.h"
#include "hwdef.h"
#include "drvtimer.h"
#include "drvheat.h"

#define _USE_INT0
// #define _USE_INT1

#define HEAT_PWM_MAX 50  // counts for 100% (each count is one half wave)

static volatile u08 uHeaterSwitchOff[2];
static s08 sPwmVal[E_SGUN_MAX];
static s08 sPwmAct = 0;
static s08 sHeatCycleEnd = 0;

static void drvheat_SwitchTriggerOffGun1( void );
static void drvheat_SwitchTriggerOffGun2( void );

void drvheat_InitModule( void )
{

//--- setup io lines for the soldering guns
  // disable heater gun control lines (set to 0)
  cbi(HEAT_PWM_PORT, HEAT_PWM_GUN1);
  cbi(HEAT_PWM_PORT, HEAT_PWM_GUN2);
  // initialize heater gun control lines to output
  sbi(HEAT_PWM_DDR, HEAT_PWM_GUN1);
  sbi(HEAT_PWM_DDR, HEAT_PWM_GUN2);
  
//--- setup io lines for the zero crossing detection
  sbi(ZEROCROSS_PORT, ZEROCROSS_IN);  // enable pull up resistor
  cbi(ZEROCROSS_DDR, ZEROCROSS_IN);   // set port as input
  
// setup IRQ for the zero crossing detection
#ifdef __AVR_ATmega32__  
  // bits MCUCR: SM2, SE, SM1, SM0, ISC11, ISC10, ISC01, ISC00
  MCUCR = 0b00000001;   // any logical change on INT0 fires anIRQ
  // bits GICR: INT1, INT0, INT2,-, -, -, IVSEL, IVCE
  GICR = 0b01000000;    // enable INT0 IRQ
#endif
#ifdef __AVR_ATmega644__
  // bits EIMSK: -, -, -, -, -, INT2, INT1, INT0
#ifdef _USE_INT0  
  EIMSK = 0b00000001;   // enable INT0
  // bits EICRA: -, -, ISC21, ISC20, ISC11, ISC10, ISC01, ISC00,
  EICRA = 0b00000001;    // any logical change on INT0 fires anIRQ
#endif  
#ifdef _USE_INT1  
  EIMSK = 0b00000010;   // enable INT1
  // bits EICRA: -, -, ISC21, ISC20, ISC11, ISC10, ISC01, ISC00,
  EICRA = 0b00000100;    // any logical change on INT1 fires anIRQ
#endif  
#endif
}

void drvheat_Task( void )
{
  static E_SOLDER_GUNS n = E_SGUN1;
  
  if( uHeaterSwitchOff[n] )
  {
    uHeaterSwitchOff[n] = 0;
    if( n == E_SGUN1 )
    {
      drvtim_TimerRemove( drvheat_SwitchTriggerOffGun1 );
      drvtim_TimerAdd( drvheat_SwitchTriggerOffGun1, 5 );
    }
    else
    {
      drvtim_TimerRemove( drvheat_SwitchTriggerOffGun2 );
      drvtim_TimerAdd( drvheat_SwitchTriggerOffGun2, 5 );
    }  
  }
  if ( n == E_SGUN2 )
    n = E_SGUN1;
  else
    n = E_SGUN2;
}

void drvheat_SetHeatingPower( E_SOLDER_GUNS uGun, u08 uPwr )
{
  cli();
  sPwmVal[uGun] = ((((u16)HEAT_PWM_MAX*10) / 100) * (u16)uPwr) / 10;
  sei();
}

u08 drvheat_GetHeatingPower( E_SOLDER_GUNS uGun )
{
  return sPwmVal[uGun] * (100/HEAT_PWM_MAX);
}

static void drvheat_SwitchTriggerOffGun1( void )
{
  cbi(HEAT_PWM_PORT,HEAT_PWM_GUN1);
}
static void drvheat_SwitchTriggerOffGun2( void )
{
  cbi(HEAT_PWM_PORT,HEAT_PWM_GUN2);
}

#ifdef _USE_INT0
// IRQ for the zero crossing detection
ISR( INT0_vect )
#endif
#ifdef _USE_INT1
// IRQ for the zero crossing detection
ISR( INT1_vect )
#endif
{
  static s08 sOffset=0;
  s08 uPwmSum_i;
  s08 uPwmAct_i;
  s08 uPwmVal1Min,uPwmVal1Max;
  s08 uPwmVal2Min,uPwmVal2Max;
  
  sPwmAct = (sPwmAct +1) % HEAT_PWM_MAX;
  uPwmAct_i = sPwmAct;
  if( uPwmAct_i == HEAT_PWM_MAX-1 )
    sHeatCycleEnd = TRUE;   // last pulse of one PWM cycle is started
  
//--- calculate summ of heating power and set the max. to 100%  
  if( ((u16)sPwmVal[0] + (u16)sPwmVal[1]) > HEAT_PWM_MAX)
  {
    if( sPwmVal[0] > sPwmVal[1] )
      sPwmVal[1] = HEAT_PWM_MAX - sPwmVal[0];  // gun1 needs more power then gun2
    else
      sPwmVal[0] = HEAT_PWM_MAX - sPwmVal[1];
  }  
  uPwmSum_i = sPwmVal[0] + sPwmVal[1];
  
  //--- begin of heating (PWM) cycle  
  if( uPwmAct_i == 0)
  {
    if( uPwmSum_i & 1 )
      sOffset ^= 1;  // toggle offset if uPwmSum is not a even number
    else
      sOffset = 0;
  }  
  
  // set pwm ranges for both guns
  uPwmVal1Min = 0;
  uPwmVal1Max = uPwmVal1Min + sPwmVal[0]; 
  uPwmVal2Min = uPwmVal1Max;
  uPwmVal2Max = uPwmVal2Min + sPwmVal[1];
  
  if( sOffset )
  {
    uPwmVal1Min++;
    uPwmVal1Max++;
    uPwmVal2Min++;
    uPwmVal2Max++;
  }
  
  // check if gun1 need power
  if( (uPwmAct_i >= uPwmVal1Min) && (uPwmAct_i < uPwmVal1Max) )
  {
    sbi(HEAT_PWM_PORT,HEAT_PWM_GUN1);  // trigger the TRIAC
  }
  if( uPwmAct_i == uPwmVal1Max-1 )  // last pulse for this cycle,
    uHeaterSwitchOff[0] = 1;      // switch off trigger output

  // check if gun2 need power
  if( (uPwmAct_i >= uPwmVal2Min) && (uPwmAct_i < uPwmVal2Max) )
  {
    sbi(HEAT_PWM_PORT,HEAT_PWM_GUN2);  // trigger the TRIAC
  }
  if( uPwmAct_i == uPwmVal2Max-1 )  // last pulse for this cycle,
    uHeaterSwitchOff[1] = 1;      // switch off trigger output
}

u08 drvheat_HeatCycleEnd( void )
{
  u08 ret = sHeatCycleEnd;
  sHeatCycleEnd = 0;
  return ret;
}




