/*
 * Copyright (c) 2008 Hans - Joachim Borchers
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/*
 * main parts of this module are from Peter Dannegger (schedule.zip)
 * Many thanks to him for this nice functions.
 */

#include "typedef.h"
#include "sysdef.h"
#include "drvtimer.h"

#define MAXTIMERS	 10
#define T_FREE		254		// > MAXTIMERS
#define T_LAST		255

typedef struct {
  uchar next;				// next in list or end mark
  t_res delta;				// delta to previous entry
  funcp func;
} t_ctrl_struct;

t_ctrl_struct t_ctrl_lst[MAXTIMERS];
t_res t_delay;				// count down until next service
uchar t_first;				// point to first entry


void drvtim_InitModule(void)
{
  uchar i;

  for( i = MAXTIMERS; i; i-- )
    t_ctrl_lst[i-1].next = T_FREE;          // mark all free
  t_first = T_LAST;                 // set no timer served
}

void drvtim_TimerTick(void)
{
  t_ctrl_struct *p;			// for faster access

  --t_delay;
  while( t_delay == 0 ){			// serve if delay = 0
    if( t_first == T_LAST )			// no function to serve
      break;
    p = &t_ctrl_lst[t_first];
    t_first = p->next;				// point to next
    p->next = T_FREE;				// mark free
    t_delay = t_ctrl_lst[t_first].delta;        // next delay delta
    p->func();				    	// execute function
  }
}


u08 drvtim_TimerAdd( funcp func, t_res delay )
{
  uchar n;
  uchar i;					// index
  uchar ipre=0;					// previous index
  t_res d1;					// last delta delay
  t_ctrl_struct *p;			// for faster access

  for( n = 0;; ){
    p = &t_ctrl_lst[n];
    if( p->next == T_FREE )
      break;
    n++;
    if( n == MAXTIMERS )
      return 1;                                 // error, list full
  }

  i = t_first;
  d1 = t_delay;
  while( i != T_LAST ){				// check until end
    if( d1 >= delay ){ 	  	      		// last >= new
      t_ctrl_lst[i].delta = d1 - delay;		// correct following entry
      break;
    }else{
      delay -= d1;				// remaining delay
      ipre = i;                         	// previous entry
      i = t_ctrl_lst[i].next;			// index of next entry
      d1 = t_ctrl_lst[i].delta; 		// next delay delta
    }
  }						// insert new entry

  p->next = i;					// following entry
  p->delta = delay;				// store remaining delay
  p->func = func;

  if( i == t_first ){ 				// insert at first
    t_first = n;
    t_delay = delay;
  }else{
    t_ctrl_lst[ipre].next = n;			// previous entry
  }
  return 0;					// successful
}


u08 drvtim_TimerRemove( funcp func )
{
  uchar ipre;					// previous index
  uchar irem=0;					// index to be removed
  uchar ifol = t_first;				// following index
  t_ctrl_struct *p;			// for faster access

  do{
    if( ifol == T_LAST )
      return 1;					// not found
    ipre = irem;
    irem = ifol;
    p = &t_ctrl_lst[irem];
    ifol = p->next;                             // get next
  }while( p->func != func );                    // found it

  p->next = T_FREE;                             // mark it as free
  if( irem == t_first ){
    t_first = ifol;                           	// serve next entry
    t_delay += t_ctrl_lst[ifol].delta;		// correct current delta
  }else{
    t_ctrl_lst[ipre].next = ifol;              	// skip index
    if( ifol != T_LAST )                   	// correct following delta
      t_ctrl_lst[ifol].delta += p->delta;
  }
  return 0;					// successful
}


